; Our model must have commands that tell it to setup the model, and to run the model
; In the interface tab, add a button; In Command, write setup
; In the interface tab, add a button; In Command, write go; Check the forever and disable until ticks start boxes

; Our model relies on the user to use Sliders to determine the number of computers, number of 
; links, and likelihood of sharing an image for our model. 
; In the interface tab, add sliders for:
; number-of-computers: min=0; increment=1; max=100; value=100
; links-to-other-computers: min=0; increment=1; max=10; value=5
; likely-to-share: min=0; increment=1; max=50; value=10; unit=%

; It would be useful to know the number of times the image has been viewed and the number of times
; the image has been shared in our simulation.
; In the interface tab, add monitors for:
; numshares and numviews
; In the interface tab, add a plot:
; Name=Image sharing; x axis label=Time; y axis label=Number
; Makes ure Auto scale? and Show legend? are checked
; Plot pens: pen name: Num. Shares --> plot numshares
; Plot pens: pen name: Num. Views --> plot numviews

globals [
  computerColors 
  ; Create a variable called numshares to count the number of times the image has been shared
  numshares
  ; Create a variable called numviews to count the number of times the image has been viewed
  numviews
]

turtles-own[
  ; Create a variable called seenimage; it will be true if the computer has seen the image
  seenimage
  ; Create a variable called sharedimage; it will be true of the computer has shared the image
  sharedimage
]

; The setup procedure
to setup
  ; Clear the world
  clear-all
  ; Initialize numshares and numviews to 0
  set numshares 0
  set numviews 0 
  ; Color the background white, and 
  ask patches [set pcolor white]
  ; Initialize an array of colors for the computers
  set computerColors [ 15 25 45 65 75 85 95 115 125 135 ] 
  ; Color the links between the computers gray
  ask links [ set color 5 ]
  ; Call the setupNetwork and setup-spatially-clustered-network procedures
  setupNetwork
  setup-spatially-clustered-network
  ; Set the sharedimage and seenimage variables to false for all of the turtles (computers)
  ask turtles [ set sharedimage false set seenimage false ]
  ; One computer must start with the image
  ; Set seenimage to true, sharedimage to false, color to black, and size to 2
  ask one-of turtles [ set seenimage true set sharedimage false set color black set size 2 ]
  ; reset the clock for the model
  reset-ticks
end

; The go procedure
to go
  ; Our program should stop when all of the computers have seen the image (when seenimage = true)
  if all? turtles [seenimage = true]
    [ stop ]
  ; To start our model, we must ask the computers to start spreading the image
  ask turtles [
    spreadimage
  ]
  ; Go to the next cycle (tick)
  tick
end

; The setupNetwork procedure
to setupNetwork
  ; set the default shape of the turtles ("house")  
  set-default-shape turtles "house"
  ; place the desired number of computers into the model
  crt number-of-computers
  [
    ; set the color of the computers to one of the colors in the array of computerColors
    set color one-of computerColors
    ; Place the computers randomly throughout the model, but not too close to the edge
    setxy (random-xcor * 0.95) (random-ycor * 0.95)
  ]
end

; The setup-spatially-clustered-network procedure
to setup-spatially-clustered-network
  let num-links (links-to-other-computers * number-of-computers) / 2
  while [count links < num-links ]
  [
    ask one-of turtles
    [
      let choice (min-one-of (other turtles with [not link-neighbor? myself])
                   [distance myself])
      if choice != nobody [ create-link-with choice ]
    ]
  ]
  ; make the network look a little prettier
  repeat 10
  [
    layout-spring turtles links 0.3 (world-width / (sqrt number-of-computers)) 1
  ]
end

; The spreadimage procedure
to spreadimage
  ; A computer can spread the image if they have seen it and not shared the image before
  if seenimage = true and sharedimage = false [
    ; A computer can spread the image only a percentage of the time
    if random 100 < likely-to-share [
      ; if this is that percentage of the time, share the image!
      ; when you share the image, call viewimage because it means that all of your neighbors will see it
      ask link-neighbors [
        viewimage
      ]
      ; You've shared the image, so set sharedimage to true
      set sharedimage true
      ; Also, increase the number of shares variable by 1
      set numshares numshares + 1
    ]
  ]
end

; The viewimage procedure
to viewimage
  ; when a computer has seen the image, change their color to black and size to 2
  set color black
  set size 2 
  ; the neighbors have seen the image, so ask them to set their seenimage variable to true
  ; This will give them a change to share the image during the next pass
  ask link-neighbors [set seenimage true]
  ; the neighbors have seen the image, so increase the number of the number of views variable by 1
  set numviews numviews + 1
end
@#$#@#$#@
GRAPHICS-WINDOW
353
10
792
470
16
16
13.0
1
10
1
1
1
0
1
1
1
-16
16
-16
16
1
1
1
ticks
30.0

BUTTON
27
23
91
56
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
104
23
167
56
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

SLIDER
27
70
199
103
number-of-computers
number-of-computers
0
100
100
1
1
NIL
HORIZONTAL

SLIDER
28
115
207
148
links-to-other-computers
links-to-other-computers
0
10
6
1
1
NIL
HORIZONTAL

SLIDER
29
161
201
194
likely-to-share
likely-to-share
0
50
10
1
1
%
HORIZONTAL

MONITOR
30
208
286
253
Number of times the image has been shared
numshares
17
1
11

PLOT
28
313
337
463
Image sharing
Time
Number
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Num. Shares" 1.0 0 -5825686 true "" "plot numshares"
"Num. Views" 1.0 0 -13840069 true "" "plot numviews"

MONITOR
29
260
286
305
Number of times the image has been viewed
numviews
17
1
11

@#$#@#$#@
## WHAT IS IT?

This model simulates the spread of an image via a social media network. One computer starts with the image and shares it. All of the computers directly linked to the original computer will see the image, and then share that image a given percentage of the time. The program will continue until all computers have viewed the image. The program will count the number of image shares and the number of times the image has been viewed.

## HOW IT WORKS

The network is created with computers and links, quantities of each are determined by the user via sliders. The image is spread when computers who have seen the image share the image a given percentage of the time, determine by the user via a slider. Computers who have already shared the image never share the image again. Every time a computer shares an image, all of the computers directly linked to that computer will see the image. 

## HOW TO USE IT

Use the sliders to set the number of computers, links, and percentage chance a computer will have of sharing the image. 

## THINGS TO NOTICE

The monitors and plot track the number of times the image has been viewed and the number of times the image has been shared. 

## THINGS TO TRY

Try increasing/decreasing the number of links between computers. Does the image spread faster or slower? How do you know? Does the number of links affect the number of shares and/or views?

Try increasing/decreasing the likelihood (percentage) that the image will be shared. Does the image spread faster or slower? How do you know? What happens to the number of shares and the number of views? How much faster or slower does the image spread?

## EXTENDING THE MODEL

Things to try: introducing another image into the model; representing two social networks - Can one image spread from one network to another?

## NETLOGO FEATURES

The code for the setupNetwork and setup-spatially-clusered-network procedures were originally used in the "Virus on a Network" model in the NetLogo Models Library.

## RELATED MODELS

Virus on a Network

## CREDITS AND REFERENCES
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.2.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment1" repetitions="10" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="1000"/>
    <metric>sharedimage</metric>
    <enumeratedValueSet variable="number-of-computers">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="links-to-other-computers">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="likely-to-share">
      <value value="10"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
